@@ -2,11 +2,10 @@ package Froody::Renderer::json;
 use strict;
 use warnings;
 
-use JSON::XS qw( to_json );
+use JSON;
 
 use Froody::Response;
 use Froody::Response::Terse;
-use Encode;
 
 # If we don't know how to do it ourselves, convert to
 # a terse and try again
@@ -19,9 +18,12 @@ use Encode;
 # and renders it with JSON
 *Froody::Response::Terse::render_json = sub {
   my $self = shift;
-  my $callback = $self->callback;
-  my $bytes = to_json( { stat => $self->status, data => $self->content } );
-  return $callback ? "$callback( $bytes )" : $bytes;
+  return objToJson({ stat => $self->status, data => $self->content });
 };
 
+use Froody::Server;
+Froody::Server->content_type_for_type(
+  json => 'text/json'
+);
+
 1;
\ No newline at end of file
