@@ -25,16 +25,14 @@ sub new {
 
   my $self = $class->SUPER::new(@_);
 
-  my @vars = $cgi->Vars();
-  my %vars = map { ref($_) || s/\xef\xbb\xbf//; $_ } @vars; # remove BOM from strings
+  my %vars = $cgi->Vars();
 
   my $method = delete $vars{method} || "";
-  $method = Encode::decode('utf-8', $method, 1 );
   $self->method($method);
 
-  my $type = Encode::decode('utf-8', delete $vars{'_type'} || delete $vars{'_froody_type'}, 1 );
+  my $type = delete $vars{'_froody_type'};
   $self->type($type);
-  
+    
   for (keys %vars) {
 
     # XXX: multiple uploads???
@@ -52,16 +50,13 @@ sub new {
       # split multi-values params into a listref
       my @vals = split("\0",$vars{$_});
       $vars{$_} = \@vals if (@vals > 1);
-      
-      # decopde params from unicode
-      my $value = Encode::decode("utf-8", delete $vars{$_}, 1 );
-      my $pname = Encode::decode("utf-8", $_, 1 );
-      $vars{ $pname } = $value;
     }
   }
 
-  if ($type and $type eq 'json') {
-    $self->callback( delete $vars{"_json_callback"} );
+  # read cookies into the request as well.
+  for ($cgi->cookie()) {
+    next unless $_ eq 'cookie_session';
+    $vars{$_} = $cgi->cookie($_);
   }
 
   $self->params(\%vars);
