@@ -12,7 +12,9 @@ sub xml {
 <spec><methods>
 <method name="foo.test.add" needslogin="0">
   <description>Add</description>
-  <arguments/>
+  <arguments>
+    <argument name="values" type="csv" optional="1" />
+  </arguments>
   <response>
     <value>1</value>
   </response>
@@ -48,6 +50,12 @@ sub xml {
   </errors>
 </method>
 
+<method name="foo.test.badhandler" needslogin="0">
+  <errors>
+    <error code="test.error"/>
+  </errors>
+</method>
+
 </methods>
 <errortypes>
   <errortype code="test.error">
